package net.osmand.plus.mapcontextmenu;

import android.graphics.drawable.Drawable;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import net.osmand.plus.R;
import net.osmand.plus.UiUtilities;
import net.osmand.plus.activities.MapActivity;
import net.osmand.plus.helpers.AndroidUiHelper;

public abstract class BaseMenuController {

	@Nullable
	private MapActivity mapActivity;
	private boolean portraitMode;
	private boolean nightMode;
	private int landscapeWidthPx;

	public BaseMenuController(@NonNull MapActivity mapActivity) {
		this.mapActivity = mapActivity;
		init();
	}

	private void init() {
		if (mapActivity != null) {
			portraitMode = AndroidUiHelper.isOrientationPortrait(mapActivity);
			landscapeWidthPx = mapActivity.getResources().getDimensionPixelSize(R.dimen.dashboard_land_width);
			updateNightMode();
		}
	}

	@Nullable
	public MapActivity getMapActivity() {
		return mapActivity;
	}

	public void setMapActivity(@Nullable MapActivity mapActivity) {
		this.mapActivity = mapActivity;
		if (mapActivity != null) {
			init();
		}
	}

	public boolean isLight() {
		return !nightMode;
	}

	protected void updateNightMode() {
		if (mapActivity != null) {
			nightMode = mapActivity.getMyApplication().getDaynightHelper().isNightModeForMapControls();
		}
	}

	public boolean isLandscapeLayout() {
		return !portraitMode;
	}

	public int getLandscapeWidthPx() {
		return landscapeWidthPx;
	}

	public float getHalfScreenMaxHeightKoef() {
		return .75f;
	}

	public int getSlideInAnimation() {
		if (isLandscapeLayout()) {
			return R.anim.slide_in_left;
		} else {
			return R.anim.slide_in_bottom;
		}
	}

	public int getSlideOutAnimation() {
		if (isLandscapeLayout()) {
			return R.anim.slide_out_left;
		} else {
			return R.anim.slide_out_bottom;
		}
	}

	protected Drawable getIconOrig(int iconId) {
		if (mapActivity != null) {
			UiUtilities iconsCache = mapActivity.getMyApplication().getUIUtilities();
			return iconsCache.getIcon(iconId, 0);
		} else {
			return null;
		}
	}

	protected Drawable getIcon(int iconId) {
		return getIcon(iconId, isLight() ? R.color.icon_color_default_light : R.color.icon_color_default_dark);
	}

	protected Drawable getIcon(int iconId, int colorId) {
		if (mapActivity != null) {
			UiUtilities iconsCache = mapActivity.getMyApplication().getUIUtilities();
			return iconsCache.getIcon(iconId, colorId);
		} else {
			return null;
		}
	}

	protected Drawable getPaintedIcon(int iconId, int color) {
		if (mapActivity != null) {
			UiUtilities iconsCache = mapActivity.getMyApplication().getUIUtilities();
			return iconsCache.getPaintedIcon(iconId, color);
		} else {
			return null;
		}
	}

	protected Drawable getIcon(int iconId, int colorLightId, int colorDarkId) {
		if (mapActivity != null) {
			UiUtilities iconsCache = mapActivity.getMyApplication().getUIUtilities();
			return iconsCache.getIcon(iconId, isLight() ? colorLightId : colorDarkId);
		} else {
			return null;
		}
	}
}
